/************
 * INCLUDES *
 ************/

// CORBA includes
#include <OB/CORBA.h>

// project includes
#include <Game_impl.h>
#include "sql_lib.h"
#include "game_constants.h"

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif


#define NO_PLAYER -1

// ----------------------------------------------------------------------
// Game_impl constructor
// ----------------------------------------------------------------------

Game_impl::Game_impl(CORBA::ORB_ptr orb, PortableServer::POA_ptr poa,
                     CORBA::ULong gid)
    : orb_(CORBA::ORB::_duplicate(orb)), 
      poa_(PortableServer::POA::_duplicate(poa))
{
  // initialize the game state
  gs.game_id = game_id = gid;
  this->init_round();

  // connect the database
  //db_connect();

  // create the database table
  //db_create_tables();
}

// ----------------------------------------------------------------------
// Game_impl Destructor
// ----------------------------------------------------------------------

Game_impl::~Game_impl() {
  //db_disconnect();
}


// ----------------------------------------------------------------------
// Game_impl private member implementation
// ----------------------------------------------------------------------

void Game_impl::init_round() {
  char map_str[NUM_ROWS * NUM_COLS + 1];

  // set the round number and remaining time
  gs.round_number = 1;           // FIXME: increment the round number
  gs.time_left    = ROUND_TIME;

  // initialize the map values
  strcpy(map_str, "002222222222200012121212121210222222222222222212121212121212222222222222222212121212121212222222222222222212121212121212222222222222222012121212121210002222222222200");
  map_str_to_array(map_str, &gs);

  // setup the players
  for (int i = 0; i < NUM_PLAYERS; i++) {
    gs.players[i].player_id = NO_PLAYER;          // FIXME -- use real player_id here
    gs.players[i].status = PSTAT_ALIVE;
    gs.players[i].bombs_dropped = 0;
    gs.players[i].rounds_won = 0;
  }
  gs.players[0].x_coord = 0;
  gs.players[0].y_coord = 0;
  gs.players[1].x_coord = NUM_COLS - 1;
  gs.players[1].y_coord = 0;
  gs.players[2].x_coord = 0;
  gs.players[2].y_coord = NUM_ROWS - 1;
  gs.players[3].x_coord = NUM_COLS - 1;
  gs.players[3].y_coord = NUM_ROWS - 1;

  gs.number_players = 0; // just started, no players yet

  // FIXME - set these values in the database
  //ds_set_gamestate(game_id, &gs);
}

struct PlayerInfo* Game_impl::get_player(unsigned int pid) {
  for (int i = 0; i < NUM_PLAYERS; i++) {
    if (gs.players[i].player_id == pid) {
      return &gs.players[i];
    }
  }
  return NULL;
}

int Game_impl::check_for_winner() {
  // FIXME: use NUM_WINS constant to check for winner
  int live_players = 0;
  struct PlayerInfo *winner = NULL;
  for(int i = 0; i < NUM_PLAYERS; i++) {
    if( !(gs.players[i].status & PSTAT_DEAD) ) {
      winner = &gs.players[i];
      live_players++;
    }
  }
  if( live_players <= 1 ){
    // Round has ended
    if( live_players == 1 ) {
      winner->rounds_won++;
    }
    gs.round_number++;
    // Reset the round
    this->init_round();
  }

  return 0;
}
	  
#define to_hash(x,y) (10000*(x) + (y))
#define x_from_hash(x) ((x)/10000)
#define y_from_hash(x) ((x)%10000)

void Game_impl::resolve_fire() {

  bomb_loc.clear();
  exploders.clear();
  
  struct PlayerInfo *pi = NULL;
  int x_coord;
  int y_coord;
  for( int i = 0; i < 4; i++ )
    {
      pi = &(gs.players[i]);
      // FIXME: Dead/Disconnected players' bombs don't explode!
      if( pi->status == PSTAT_ALIVE )
	{
	  for( int j = 0; j < pi->bombs_dropped; j++ )
	    {
	      x_coord = pi->bombs[j].x_coord;
	      y_coord = pi->bombs[j].y_coord;
	      if( pi->bombs[j].status = READY2BLOW )
		{
		  pi->bombs[j].status = EXPLODED;
		  pi->bombs_dropped -= 1;
		  assert( pi->bombs_dropped >= 0);
		  // Add bomb to list of bombs to explode
		  exploders.push_back( &(pi->bombs[j]) );
		}
	      bomb_loc[ to_hash(x_coord, y_coord) ] = &(pi->bombs[j]);
	    }
	}
    }
  
  if ( exploders.empty() )
    return;

  for( exploders_iter = exploders.begin(); 
       exploders_iter != exploders.end(); 
       exploders_iter++ )
    {
      // Make sure this bomb doesn't recursively set itself off.

      bomb_loc.erase( to_hash( (*exploders_iter)->x_coord,
			       (*exploders_iter)->y_coord ) );
      
      resolve_fire_radius( FIRE_RADIUS, MOVE_UP );
      resolve_fire_radius( FIRE_RADIUS, MOVE_DOWN );
      resolve_fire_radius( FIRE_RADIUS, MOVE_LEFT );
      resolve_fire_radius( FIRE_RADIUS, MOVE_RIGHT );
    }

}

void Game_impl::resolve_fire_radius( int radius, int direction )
{
  int x_movement;
  int y_movement;

  switch( direction )
    {
    case MOVE_UP:
      x_movement = 0;
      y_movement = -1;
      break;
    case MOVE_DOWN:
      x_movement = 0;
      y_movement = 1;
      break;
    case MOVE_RIGHT:
      x_movement = 1;
      y_movement = 0;
      break;
    case MOVE_LEFT:
      x_movement = -1;
      y_movement = 0;
      break;
    default:
      // Oops.
      return;
    };

  int x_coord = (*exploders_iter)->x_coord;
  int y_coord = (*exploders_iter)->y_coord;
  for( int i = 0; i < radius; i++ )
    {
      x_coord += x_movement;
      y_coord += y_movement;

      // Corner cases: can't bomb off the map.
      if( x_coord < 0 || x_coord >= NUM_COLS )
	break;
      if( y_coord < 0 || y_coord >= NUM_ROWS )
	break;

      long int *map_point = &( gs.map_info[y_coord][x_coord] );

      if( *map_point == MAP_EMPTY )
	{
	  // See if a hapless player is standing in the line of fire
	  for( int j = 0; j < 4; j++ )
	    {
	      if( gs.players[j].x_coord == x_coord &&
		  gs.players[j].y_coord == y_coord )
		{
		  gs.players[j].status |= PSTAT_DEAD;
		}
	    }
	  // See if this bomb set off other bombs
	  bomb_loc_iter = bomb_loc.find( to_hash( x_coord, y_coord ) );
	  if( bomb_loc_iter != bomb_loc.end() )
	    {
	      // Push new bomb onto stack of exploders
	      if( bomb_loc_iter->second->status == UNEXPLODED )
		{
		  bomb_loc_iter->second->status == EXPLODED;
		  exploders.push_back( bomb_loc_iter->second );
		}
	      break;
	    }
	  else
	    {
	      continue;
	    }
	}
      else if( *map_point == MAP_WALL )
	{
	  break;
	}
      else if( *map_point == MAP_BRICK )
	{
	  // Brick is destroyed
	  *map_point == MAP_EMPTY;
	  break;
	}
    }

}
	 


// ----------------------------------------------------------------------
// Game_impl public member implementation
// ----------------------------------------------------------------------

struct GameState
Game_impl::update(const struct ClientAction &c_action)
    throw(CORBA::SystemException)
{
  static unsigned long seq = 0;

  // FIXME: Get Gamestate from database
  //db_get_gamestate(game_id, &gs);
  
  struct PlayerInfo *pi = get_player(c_action.player_id);

  if (pi == NULL) {
    cerr << "ERROR: INVALID PLAYER ID";
    return gs;
  }
  
  unsigned int x   = pi->x_coord;
  unsigned int y   = pi->y_coord;
  unsigned int pid = pi->player_id;

  // fill in the GameState structure to send back to the client
  gs.seq_num = seq++;

  // plant a bomb if necessary
  if (c_action.request_bomb) {
    // Fixme: Can currently overwrite other bombs.
    if (pi->bombs_dropped < MAX_BOMBS) {
      pi->bombs[pi->bombs_dropped].player_id = pid;
      pi->bombs[pi->bombs_dropped].x_coord = x;
      pi->bombs[pi->bombs_dropped].y_coord = y;
      pi->bombs_dropped++;
    }
  }

  // handle the player movement
  switch (c_action.arrow) {
    case MOVE_LEFT:
      if ((x > 0) && (gs.map_info[y][x-1] == MAP_EMPTY)) {
        pi->x_coord = pi->x_coord - 1;
      }
      break;
    case MOVE_RIGHT:
      if ((x < NUM_COLS-1) && (gs.map_info[y][x+1] == MAP_EMPTY)) {
        pi->x_coord = pi->x_coord + 1;
      }
      break;
    case MOVE_UP:
      if ((y > 0) && (gs.map_info[y-1][x] == MAP_EMPTY)) {
        pi->y_coord = pi->y_coord - 1;
      }
      break;
    case MOVE_DOWN:
      if ((y < NUM_ROWS-1) && (gs.map_info[y+1][x] == MAP_EMPTY)) {
        pi->y_coord = pi->y_coord + 1;
      }
      break;
  }

  //this->resolve_fire();
  //this->check_for_winner();

  // FIXME: write the gs values back to the database
  //db_set_gamestate(game_id, &gs);

  return gs;
}

long Game_impl::exit_game(CORBA::ULong player_id, CORBA::ULong game_id)
  throw(CORBA::SystemException)
{
  // FIX => db_exit_game is having a segmentation fault error
  // update the datebase for leaving player
  //int flag = db_exit_game((int)game_id, (int)player_id);

  // get playerInfo
  struct PlayerInfo *pi = get_player( player_id );

  if (pi == NULL) {
    cerr << "ERROR: INVALID PLAYER ID";
    return 0;
  }
  
  // Set player status
  pi->status = PSTAT_LEFTGAME;
  pi->player_id = NO_PLAYER;
  gs.number_players--;

  printf("Player %d from Game %d left the game\n",(int)player_id,(int)game_id);

  /*
  int flag = -2;

  // Check if I need to remove the game as well
  for (int i = 0; i < NUM_PLAYERS; i++) {
    if (gs.players[i].player_id != NO_PLAYER) {
      flag = 0;
    }
  }
  */

  // If this is the last player in the game, deactivate this game
  if(gs.number_players == 0)
    {
      // Get the default POA to associate this object to
      PortableServer::POA_var poa = _default_POA();

      char gameIdStr[50];
      sprintf(gameIdStr,"%d",(int)game_id);
      PortableServer::ObjectId_var gameId = PortableServer::string_to_ObjectId(gameIdStr);
      poa -> deactivate_object(gameId); 
      printf("Game %d deactivated\n",(int)game_id);
    }

  return (long)player_id;
}

bool Game_impl::is_game_started(CORBA::ULong player_id, CORBA::ULong game_id)
  throw(CORBA::SystemException)
{
  //printf("numplayer %d\n",gs.number_players);
  if(gs.number_players == NUM_PLAYERS)
    return true;
  else
    return false;
}

PortableServer::POA_ptr
Game_impl::_default_POA()
{
    return PortableServer::POA::_duplicate(poa_);
}

long Game_impl::get_id()
  throw(CORBA::SystemException)
{
  return gs.game_id;
}


// ----------------------------------------------------------------------
// Insert new player into the game, this function is called
// by join_game function when the client logs into the server for the
// first time
// This function will return player_number={0,1,2,..,NUM_PLAYERS} if the player 
// is succesffully added to the game and return FAILED if the player failed 
// to be added to the game
// FIX = when the DB is done, this should be done in the DB instead
// ----------------------------------------------------------------------
long Game_impl::insert_player(CORBA::ULong pid) 
  throw(CORBA::SystemException)
{
  for (int i = 0; i < NUM_PLAYERS; i++) {
    if (gs.players[i].player_id == NO_PLAYER) {
      gs.players[i].player_id = pid;
      gs.number_players++;
      return i;
    }
  }
  return FAILED;
}
