#include <mysql.h> /* Headers for MySQL usage */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "sql_lib.h"
#include "game_constants.h"


/***********
 * GLOBALS *
 ***********/

static MYSQL demo_db;
static char *host = "mysql.ece.cmu.edu";
static char *username = "c18749t7";
static char *password = NULL; //"g7&bor4ba";


/*************
 * FUNCTIONS *
 *************/

int connect_to_database()
{
  /* Make connection with mysql_connect(MYSQL db, char *host, char *username,
     char *password */
  if(!mysql_connect(&demo_db, host, username, password)){ 
    printf(mysql_error(&demo_db));
    return -1;
  }

  /* Select the database we want to use */
  if(mysql_select_db(&demo_db, username)){
    printf(mysql_error(&demo_db));
    return -1;
  }

  return 0;
}

int disconnect_database()
{
  mysql_close(&demo_db);
  return 0;
}

int create_tables()
{
  char query[255];

  // Create GameState table
  // Build query
  sprintf(query,
	  "CREATE TABLE GameState ("
	  "id int NOT NULL AUTO_INCREMENT PRIMARY KEY, "
	  "status int, "
	  "number_players int, "
	  "round_number int, "
	  "time_left int");
  printf("query: %s\n", query);

  // Make query
  if(mysql_real_query(&demo_db, query, strlen(query))){
    printf(mysql_error(&demo_db));
    return -1;
  }
  // Create Player table
  sprintf(query, 
	  "CREATE TABLE Player ("
	  "id int NOT NULL AUTO_INCREMENT PRIMARY KEY, "
	  "game_id int, "
	  "status int, "
	  "x_coord int, "
	  "y_coord int, "
	  "bombs_dropped int, "
	  "bombs_max int, "
	  "rounds_won int, "
	  "FOREIGN KEY (game_id) REFERENCES GameState(id))");
  printf("query: %s\n", query);

  if(mysql_real_query(&demo_db, query, strlen(query))){
    printf(mysql_error(&demo_db));
    return -1;
  }

  // Create Bomb table
  sprintf(query,
	  "CREATE TABLE Bomb ("
	  "id int NOT NULL AUTO_INCREMENT PRIMARY KEY, "
	  "player_id int, "
	  "x_coord int, "
	  "y_coord int, "
	  "timeleft int, "
	  "FOREIGN KEY (player_id) REFERENCES Player(id)");
  printf("query: %s\n", query);

  if(mysql_real_query(&demo_db, query, strlen(query))){
    printf(mysql_error(&demo_db));
    return -1;
  }

  // Create Map table
  sprintf(query,
	  "CREATE TABLE Map ("
	  "game_id int, "
	  "mapstring VARCHAR(15*12+1), "
	  "FOREIGN KEY (game_id) REFERENCES GameState(id)");
  printf("query: %s\n", query);

  if(mysql_real_query(&demo_db, query, strlen(query))){
    printf(mysql_error(&demo_db));
    return -1;
  }
  return 0;
}

int check_games()
{
  char query[255];
  int game_id;
  MYSQL_RES *res;
  MYSQL_ROW row;

  // status == 1 means game is GSTATE_WAITING for players to join
  sprintf(query,
	  "SELECT id FROM GameState "
	  "WHERE number_players < %d AND status = 1 "
	  "ORDER BY id DESC"
	  "LIMIT 1", NUM_PLAYERS);
  printf("query: %s\n", query);

  if(mysql_real_query(&demo_db, query, strlen(query))){
    printf(mysql_error(&demo_db));
    return -1;
  }

  res = mysql_store_result(&demo_db); // Download result from server
  row = mysql_fetch_row(res); // Get a row from the results

  if(row != NULL)
    game_id = (int) row[0];
  else
    game_id = -1;

  mysql_free_result(res); // Release memory used to store results.
  printf("Game_id = %d\n", game_id);

  return game_id;
}

int create_game()
{
  char query[255];
  int insert_id;
  
  // FIX: DUMMY VALUES
  sprintf(query,
	  "INSERT INTO GameState "
	  "(status, number_players, round_number, time_left) "
	  "VALUES ('%d', 0, 0, 0)", GSTAT_WAITING);
  printf("query: %s\n", query);

  if(mysql_real_query(&demo_db, query, strlen(query))){
    printf(mysql_error(&demo_db));
    return -1;
  }

  // Find what id the new player was given
  insert_id = mysql_insert_id(&demo_db);

  return insert_id;
}

int create_player(int game_id, int x_coord, int y_coord)
{
  char query[255];
  int insert_id;

  begin_transaction();
  
  // Create a new player
  sprintf(query, 
	  "INSERT INTO Player "
	  "(game_id, status, x_coord, y_coord, bombs_dropped, "
	  "bombs_max, rounds_won) "
	  "VALUES ('%d', '%d', '%d', '%d', '%d', '%d')",
	  game_id, PSTAT_ALIVE, x_coord, y_coord, 0, MAX_BOMBS, 0);
  printf("query: %s\n", query);

  if(mysql_real_query(&demo_db, query, strlen(query))){
    rollback_transaction();
    printf(mysql_error(&demo_db));
    return -1;
  }

  // Find what id the new player was given
  insert_id = mysql_insert_id(&demo_db);

  // NEED TO CHECK IF MAX NUM PLAYERS HAS BEEN REACHED
  // Add him to the game_id
  sprintf(query,
	  "UPDATE GameState SET number_players = number_players+1 "
	  "WHERE id = %d", game_id);

  if(mysql_real_query(&demo_db, query, strlen(query))){
    rollback_transaction();
    printf(mysql_error(&demo_db));
    return -1;
  }

  commit_transaction();

  return insert_id;
}

int move_player(int game_id, int player_id, int x_coord, int y_coord)
{
  char query[255];

  sprintf(query,
	  "UPDATE Player "
	  "SET game_id = %d, x_coord = %d, y_coord = %d "
	  "WHERE id = %d", 
	  game_id, x_coord, y_coord, player_id);

  if(mysql_real_query(&demo_db, query, strlen(query))){
    printf(mysql_error(&demo_db));
    return -1;
  }

  return 0;
}

int plant_bomb(int game_id, int player_id, int x_coord, int y_coord)
{
  char query[255];

  begin_transaction(); 

  // FIX: NEED TO GET TIME FROM MYSQL AND STORE INTO BOMB
  sprintf(query,
	  "INSERT INTO Bomb (player_id, x_coord, y_coord, timeleft) "
	  "VALUES (%d, %d, %d, 0)",
	  player_id, x_coord, y_coord);

  if(mysql_real_query(&demo_db, query, strlen(query))){
    rollback_transaction();
    printf(mysql_error(&demo_db));
    return -1;
  }

  sprintf(query,
	  "UPDATE Player SET bombs_dropped = bombs_dropped + 1 "
	  "WHERE player_id = %d",
	  player_id);

  if(mysql_real_query(&demo_db, query, strlen(query))){
    rollback_transaction();
    printf(mysql_error(&demo_db));
    return -1;
  }

  commit_transaction();

  // DEBUG
  // Find what id the new bomb was given
  int insert_id = mysql_insert_id(&demo_db);
  printf("Bomb with id %d was inserted\n", insert_id);

  return 0;

}

// FIX: do these transactions need to rollback if fail?
int begin_transaction()
{
  char query[255];

  sprintf(query, "BEGIN");

  if(mysql_real_query(&demo_db, query, strlen(query))){
    printf(mysql_error(&demo_db));
    return -1;
  }

  return 0;

}

int commit_transaction()
{
  char query[255];

  sprintf(query, "COMMIT");

  if(mysql_real_query(&demo_db, query, strlen(query))){
    printf(mysql_error(&demo_db));
    return -1;
  }

  return 0;
}

int rollback_transaction()
{
  char query[255];

  sprintf(query, "ROLLBACK");

  if(mysql_real_query(&demo_db, query, strlen(query))){
    printf(mysql_error(&demo_db));
    return -1;
  }

  return 0;
}



// OLD STUFF

/*
 // Check if the maximum number of bombs has already been planted
  sprintf(query,
	  "SELECT bombs_dropped FROM Player WHERE id = %d"
	  player_id);

  if(mysql_real_query(&demo_db, query, strlen(query))){
    rollback_transaction();
    printf(mysql_error(&demo_db));
    return -1;
  }

  res = mysql_store_result(&demo_db); // Download result from server
  row = mysql_fetch_row(res); // Get a row from the results

  int bombs_dropped = row[5];
  int bombs_max = row[6];

  printf("Player %d  has planted %d bombs (max: %d)\n", 
	 player_id, bombs_dropped, bombs_max);
  mysql_free_result(res); // Release memory used to store results.
 
  if(bombs_dropped >= bombs_max){
*/

/*
int update_table(int id, int value)
{
  char query[255];

  sprintf(query, "UPDATE borba_table SET value = %d WHERE id = %d", value, id);
  printf("query: %s\n", query);

  if(mysql_real_query(&demo_db, query, strlen(query))){
    printf(mysql_error(&demo_db));
    return -1;
  }

  return 0;
}

int borba_insert(int id, int value)
{
  char query[255];
  int insert_id;
  MYSQL_RES *res;
  MYSQL_ROW row;

  sprintf(query, "INSERT INTO borba_table(id, value) VALUES ('%d', '%d')", 
	  id, value);
  printf("query: %s\n", query);

  if(mysql_real_query(&demo_db, query, strlen(query))){
    printf(mysql_error(&demo_db));
    return -1;
  }

  // this function actually increments id automatically, but we need to set
  // our database flag AUTO_INCREMENT (not set now)
  insert_id=mysql_insert_id(&demo_db); // Find what id that data was given

  query = malloc(255);
  //sprintf(query, "SELECT demodata FROM demotable WHERE id='%d'", insert_id);
  sprintf(query, "SELECT id, value FROM borba_table WHERE id='%d'", id);
  if(mysql_real_query(&demo_db, query, strlen(query))){ // Make query
     printf(mysql_error(&demo_db));
     exit(1);
  }
  free(query);

  res=mysql_store_result(&demo_db); // Download result from server
  row=mysql_fetch_row(res); // Get a row from the results
  printf("You inserted '%s', '%s'.\n", row[0], row[1]);
  mysql_free_result(res); // Release memory used to store results.

  return 0;
}
*/ 
